@@ -48,14 +48,14 @@ staticforward PyTypeObject Auth_Type;
 
 static bool ra_check_svn_path(char *path)
 {
-    /* svn_ra_check_path will raise an assertion error if the path has a
-     * leading '/'. Raise a Python exception if there ar eleading '/'s so that 
+	/* svn_ra_check_path will raise an assertion error if the path has a
+	 * leading '/'. Raise a Python exception if there ar eleading '/'s so that 
 	 * the Python interpreter won't crash and die. */
-    if (*path == '/') {
-        PyErr_SetString(PyExc_ValueError, "invalid path has a leading '/'");
-        return true;
-    }
-    return false;
+	if (*path == '/') {
+		PyErr_SetString(PyExc_ValueError, "invalid path has a leading '/'");
+		return true;
+	}
+	return false;
 }
 
 static svn_error_t *py_commit_callback(const svn_commit_info_t *commit_info, void *baton, apr_pool_t *pool)
@@ -141,10 +141,16 @@ static PyObject *reporter_set_path(PyObject *self, PyObject *args)
 	svn_depth_t depth = svn_depth_infinity;
 	ReporterObject *reporter = (ReporterObject *)self;
 
-	if (!PyArg_ParseTuple(args, "slb|zi", &path, &revision, &start_empty, 
+	if (!PyArg_ParseTuple(args, "slb|zi:set_path", &path, &revision, &start_empty, 
 						  &lock_token, &depth))
 		return NULL;
 
+	if (reporter->ra == NULL) {
+		PyErr_SetString(PyExc_RuntimeError,
+			"Reporter already finished.");
+		return NULL;
+	}
+
 #if ONLY_SINCE_SVN(1, 5)
 	RUN_SVN(reporter->reporter->set_path(reporter->report_baton, 
 												  path, revision, depth, start_empty, 
@@ -166,11 +172,17 @@ static PyObject *reporter_delete_path(PyObject *self, PyObject *args)
 {
 	ReporterObject *reporter = (ReporterObject *)self;
 	char *path;
-	if (!PyArg_ParseTuple(args, "s", &path))
+	if (!PyArg_ParseTuple(args, "s:delete_path", &path))
 		return NULL;
 
-	RUN_SVN(reporter->reporter->delete_path(reporter->report_baton, 
-													path, reporter->pool));
+	if (reporter->ra == NULL) {
+		PyErr_SetString(PyExc_RuntimeError,
+			"Reporter already finished.");
+		return NULL;
+	}
+
+	RUN_SVN(reporter->reporter->delete_path(reporter->report_baton,
+		path, reporter->pool));
 
 	Py_RETURN_NONE;
 }
@@ -184,9 +196,16 @@ static PyObject *reporter_link_path(PyObject *self, PyObject *args)
 	ReporterObject *reporter = (ReporterObject *)self;
 	svn_depth_t depth = svn_depth_infinity;
 
-	if (!PyArg_ParseTuple(args, "sslb|zi", &path, &url, &revision, &start_empty, &lock_token, &depth))
+	if (!PyArg_ParseTuple(args, "sslb|zi:kink_path", &path, &url, &revision,
+			&start_empty, &lock_token, &depth))
 		return NULL;
 
+	if (reporter->ra == NULL) {
+		PyErr_SetString(PyExc_RuntimeError,
+			"Reporter already finished.");
+		return NULL;
+	}
+
 #if ONLY_SINCE_SVN(1, 5)
 	RUN_SVN(reporter->reporter->link_path(reporter->report_baton, path, url, 
 				revision, depth, start_empty, lock_token, reporter->pool));
@@ -207,12 +226,20 @@ static PyObject *reporter_finish(PyObject *self)
 {
 	ReporterObject *reporter = (ReporterObject *)self;
 
+	if (reporter->ra == NULL) {
+		PyErr_SetString(PyExc_RuntimeError,
+			"Reporter already finished.");
+		return NULL;
+	}
+
 	reporter->ra->busy = false;
 
-	RUN_SVN(reporter->reporter->finish_report(reporter->report_baton, 
-													  reporter->pool));
+	RUN_SVN(reporter->reporter->finish_report(
+		reporter->report_baton, reporter->pool));
 
+	apr_pool_destroy(reporter->pool);
 	Py_XDECREF(reporter->ra);
+	reporter->ra = NULL;
 
 	Py_RETURN_NONE;
 }
@@ -221,12 +248,20 @@ static PyObject *reporter_abort(PyObject *self)
 {
 	ReporterObject *reporter = (ReporterObject *)self;
 
+	if (reporter->ra == NULL) {
+		PyErr_SetString(PyExc_RuntimeError,
+			"Reporter already finished.");
+		return NULL;
+	}
+
 	reporter->ra->busy = false;
 
 	RUN_SVN(reporter->reporter->abort_report(reporter->report_baton, 
 													 reporter->pool));
 
+	apr_pool_destroy(reporter->pool);
 	Py_XDECREF(reporter->ra);
+	reporter->ra = NULL;
 
 	Py_RETURN_NONE;
 }
@@ -250,8 +285,11 @@ static PyMethodDef reporter_methods[] = {
 static void reporter_dealloc(PyObject *self)
 {
 	ReporterObject *reporter = (ReporterObject *)self;
-	/* FIXME: Warn the user if abort_report/finish_report wasn't called? */
-	apr_pool_destroy(reporter->pool);
+	if (reporter->ra != NULL) {
+		/* FIXME: Warn */
+		apr_pool_destroy(reporter->pool);
+		Py_DECREF(reporter->ra);
+	}
 	PyObject_Del(self);
 }
 
@@ -386,15 +424,13 @@ static svn_error_t *py_ra_file_rev_handler(void *baton, const char *path, svn_re
 
 #endif
 
-
-
 static void ra_done_handler(void *_ra)
 {
 	RemoteAccessObject *ra = (RemoteAccessObject *)_ra;
 
 	ra->busy = false;
 
-	Py_XDECREF(ra);
+	Py_DECREF(ra);
 }
 
 #define RUN_RA_WITH_POOL(pool, ra, cmd) { \
@@ -655,7 +691,7 @@ static PyObject *ra_reparent(PyObject *self, PyObject *args)
 	apr_pool_t *temp_pool;
 	RemoteAccessObject *ra = (RemoteAccessObject *)self;
 
-	if (!PyArg_ParseTuple(args, "s", &url))
+	if (!PyArg_ParseTuple(args, "s:reparent", &url))
 		return NULL;
 
 	if (ra_check_busy(ra))
@@ -847,7 +883,7 @@ static PyObject *ra_get_url(PyObject *self, void *closure)
 static PyObject *ra_do_update(PyObject *self, PyObject *args)
 {
 	svn_revnum_t revision_to_update_to;
-	char *update_target; 
+	char *update_target;
 	bool recurse;
 	PyObject *update_editor;
 	const REPORTER_T *reporter;
@@ -886,12 +922,11 @@ static PyObject *ra_do_update(PyObject *self, PyObject *args)
 		return NULL;
 	}
 	Py_BEGIN_ALLOW_THREADS
-	err = svn_ra_do_update(ra->ra, &reporter, 
-												  &report_baton, 
-												  revision_to_update_to, 
-												  update_target, recurse, 
-												  &py_editor, update_editor, 
-												  temp_pool);
+	err = svn_ra_do_update(ra->ra, &reporter,
+		&report_baton, revision_to_update_to,
+		update_target, recurse,
+		&py_editor, update_editor,
+		temp_pool);
 
 #endif
 	Py_END_ALLOW_THREADS
@@ -918,9 +953,9 @@ static PyObject *ra_do_switch(PyObject *self, PyObject *args)
 {
 	RemoteAccessObject *ra = (RemoteAccessObject *)self;
 	svn_revnum_t revision_to_update_to;
-	char *update_target; 
+	char *update_target;
 	bool recurse;
-	char *switch_url; 
+	char *switch_url;
 	PyObject *update_editor;
 	const REPORTER_T *reporter;
 	void *report_baton;
@@ -935,8 +970,11 @@ static PyObject *ra_do_switch(PyObject *self, PyObject *args)
 		return NULL;
 
 	temp_pool = Pool(NULL);
-	if (temp_pool == NULL)
+	if (temp_pool == NULL) {
+		ra->busy = false;
 		return NULL;
+	}
+
 	Py_INCREF(update_editor);
 	Py_BEGIN_ALLOW_THREADS
 
@@ -964,8 +1002,11 @@ static PyObject *ra_do_switch(PyObject *self, PyObject *args)
 		return NULL;
 	}
 	ret = PyObject_New(ReporterObject, &Reporter_Type);
-	if (ret == NULL)
+	if (ret == NULL) {
+		apr_pool_destroy(temp_pool);
+		ra->busy = false;
 		return NULL;
+	}
 	ret->reporter = reporter;
 	ret->report_baton = report_baton;
 	ret->pool = temp_pool;
@@ -1038,8 +1079,6 @@ static PyObject *ra_do_diff(PyObject *self, PyObject *args)
 	return (PyObject *)ret;
 }
 
-
-
 static PyObject *ra_replay(PyObject *self, PyObject *args)
 {
 	RemoteAccessObject *ra = (RemoteAccessObject *)self;
@@ -1048,7 +1087,7 @@ static PyObject *ra_replay(PyObject *self, PyObject *args)
 	PyObject *update_editor;
 	bool send_deltas = true;
 
-	if (!PyArg_ParseTuple(args, "llO|b", &revision, &low_water_mark, &update_editor, &send_deltas))
+	if (!PyArg_ParseTuple(args, "llO|b:replay", &revision, &low_water_mark, &update_editor, &send_deltas))
 		return NULL;
 
 	if (ra_check_busy(ra))
@@ -1118,7 +1157,7 @@ static PyObject *ra_replay_range(PyObject *self, PyObject *args)
 	PyObject *cbs;
 	bool send_deltas = true;
 
-	if (!PyArg_ParseTuple(args, "lllO|b", &start_revision, &end_revision, &low_water_mark, &cbs, &send_deltas))
+	if (!PyArg_ParseTuple(args, "lllO|b:replay_range", &start_revision, &end_revision, &low_water_mark, &cbs, &send_deltas))
 		return NULL;
 
 	if (!PyTuple_Check(cbs)) {
@@ -1157,7 +1196,7 @@ static PyObject *ra_rev_proplist(PyObject *self, PyObject *args)
 	RemoteAccessObject *ra = (RemoteAccessObject *)self;
 	svn_revnum_t rev;
 	PyObject *py_props;
-	if (!PyArg_ParseTuple(args, "l", &rev))
+	if (!PyArg_ParseTuple(args, "l:rev_proplist", &rev))
 		return NULL;
 
 	if (ra_check_busy(ra))
@@ -1191,7 +1230,8 @@ static PyObject *get_commit_editor(PyObject *self, PyObject *args, PyObject *kwa
 #endif
 	svn_error_t *err;
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O|OOb", kwnames, &revprops, &commit_callback, &lock_tokens, &keep_locks))
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O|OOb:get_commit_editor",
+		kwnames, &revprops, &commit_callback, &lock_tokens, &keep_locks))
 		return NULL;
 
 	pool = Pool(NULL);
@@ -1287,7 +1327,8 @@ static PyObject *ra_change_rev_prop(PyObject *self, PyObject *args)
 	apr_pool_t *temp_pool;
 	svn_string_t *val_string;
 
-	if (!PyArg_ParseTuple(args, "lss#", &rev, &name, &value, &vallen))
+	if (!PyArg_ParseTuple(args, "lss#:change_rev_prop", &rev, &name, &value,
+			&vallen))
 		return NULL;
 	if (ra_check_busy(ra))
 		return NULL;
@@ -1321,7 +1362,7 @@ static PyObject *ra_get_dir(PyObject *self, PyObject *args, PyObject *kwargs)
 	PyObject *py_dirents, *py_props;
     char *kwnames[] = { "path", "revision", "fields", NULL };
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|lI", kwnames,
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|lI:get_dir", kwnames,
                                      &path, &revision, &dirent_fields))
 		return NULL;
 
@@ -1380,7 +1421,7 @@ static PyObject *ra_get_file(PyObject *self, PyObject *args)
 	PyObject *py_stream, *py_props;
 	apr_pool_t *temp_pool;
 
-	if (!PyArg_ParseTuple(args, "sO|l", &path, &py_stream, &revision))
+	if (!PyArg_ParseTuple(args, "sO|l:get_file", &path, &py_stream, &revision))
 		return NULL;
 
 	if (ra_check_busy(ra))
@@ -1407,7 +1448,7 @@ static PyObject *ra_get_file(PyObject *self, PyObject *args)
 	}
 
 	apr_pool_destroy(temp_pool);
-		 
+
 	return Py_BuildValue("(lN)", fetch_rev, py_props);
 }
 
@@ -1418,7 +1459,7 @@ static PyObject *ra_get_lock(PyObject *self, PyObject *args)
 	svn_lock_t *lock;
 	apr_pool_t *temp_pool;
 
-	if (!PyArg_ParseTuple(args, "s", &path))
+	if (!PyArg_ParseTuple(args, "s:get_lock", &path))
 		return NULL;
 
 	if (ra_check_busy(ra))
@@ -1441,7 +1482,7 @@ static PyObject *ra_check_path(PyObject *self, PyObject *args)
 	svn_node_kind_t kind;
 	apr_pool_t *temp_pool;
 
-	if (!PyArg_ParseTuple(args, "sl", &path, &revision))
+	if (!PyArg_ParseTuple(args, "sl:check_path", &path, &revision))
 		return NULL;
 	if (ra_check_svn_path(path))
 		return NULL;
@@ -1468,7 +1509,7 @@ static PyObject *ra_stat(PyObject *self, PyObject *args)
 	svn_dirent_t *dirent;
 	apr_pool_t *temp_pool;
 
-	if (!PyArg_ParseTuple(args, "sl", &path, &revision))
+	if (!PyArg_ParseTuple(args, "sl:stat", &path, &revision))
 		return NULL;
 	if (ra_check_svn_path(path))
 		return NULL;
@@ -1495,7 +1536,7 @@ static PyObject *ra_has_capability(PyObject *self, PyObject *args)
 	RemoteAccessObject *ra = (RemoteAccessObject *)self;
 	int has = 0;
 
-	if (!PyArg_ParseTuple(args, "s", &capability))
+	if (!PyArg_ParseTuple(args, "s:has_capability", &capability))
 		return NULL;
 
 	if (ra_check_busy(ra))
@@ -1523,7 +1564,7 @@ static PyObject *ra_unlock(PyObject *self, PyObject *args)
 	apr_pool_t *temp_pool;
 	apr_hash_t *hash_path_tokens;
 
-	if (!PyArg_ParseTuple(args, "ObO", &path_tokens, &break_lock, &lock_func))
+	if (!PyArg_ParseTuple(args, "ObO:unlock", &path_tokens, &break_lock, &lock_func))
 		return NULL;
 
 	if (ra_check_busy(ra))
@@ -1555,7 +1596,7 @@ static PyObject *ra_lock(PyObject *self, PyObject *args)
 	svn_revnum_t *rev;
 	Py_ssize_t idx = 0;
 
-	if (!PyArg_ParseTuple(args, "OsbO", &path_revs, &comment, &steal_lock, 
+	if (!PyArg_ParseTuple(args, "OsbO:lock", &path_revs, &comment, &steal_lock,
 						  &lock_func))
 		return NULL;
 
@@ -1599,7 +1640,7 @@ static PyObject *ra_get_locks(PyObject *self, PyObject *args)
 	svn_lock_t *lock;
 	PyObject *ret;
 
-	if (!PyArg_ParseTuple(args, "s", &path))
+	if (!PyArg_ParseTuple(args, "s:get_locks", &path))
 		return NULL;
 
 	if (ra_check_svn_path(path))
@@ -1646,7 +1687,7 @@ static PyObject *ra_get_locations(PyObject *self, PyObject *args)
 	apr_ssize_t klen;
 	char *val;
 
-	if (!PyArg_ParseTuple(args, "slO", &path, &peg_revision, &location_revisions))
+	if (!PyArg_ParseTuple(args, "slO:get_locations", &path, &peg_revision, &location_revisions))
 		return NULL;
 
 	if (ra_check_svn_path(path))
@@ -1741,7 +1782,7 @@ static PyObject *ra_mergeinfo(PyObject *self, PyObject *args)
 	svn_mergeinfo_inheritance_t inherit = svn_mergeinfo_explicit;
 	svn_boolean_t include_descendants;
 
-	if (!PyArg_ParseTuple(args, "O|lib", &paths, &revision, &inherit, &include_descendants))
+	if (!PyArg_ParseTuple(args, "O|lib:mergeinfo", &paths, &revision, &inherit, &include_descendants))
 		return NULL;
 
 	temp_pool = Pool(NULL);
@@ -1807,8 +1848,8 @@ static PyObject *ra_get_location_segments(PyObject *self, PyObject *args)
 	PyObject *py_rcvr;
 	apr_pool_t *temp_pool;
 
-	if (!PyArg_ParseTuple(args, "slllO", &path, &peg_revision, &start_revision, 
-						  &end_revision, &py_rcvr))
+	if (!PyArg_ParseTuple(args, "slllO:get_location_segments", &path,
+			&peg_revision, &start_revision, &end_revision, &py_rcvr))
 		return NULL;
 
 	if (ra_check_svn_path(path))
@@ -1841,8 +1882,8 @@ static PyObject *ra_get_file_revs(PyObject *self, PyObject *args)
 	RemoteAccessObject *ra = (RemoteAccessObject *)self;
 	svn_boolean_t include_merged_revisions = FALSE;
 
-	if (!PyArg_ParseTuple(args, "sllO|b", &path, &start, &end, &file_rev_handler, 
-						  &include_merged_revisions))
+	if (!PyArg_ParseTuple(args, "sllO|b:get_file_revs", &path, &start,
+			&end, &file_rev_handler, &include_merged_revisions))
 		return NULL;
 
 	if (ra_check_svn_path(path))
@@ -2084,6 +2125,7 @@ static void auth_provider_dealloc(PyObject *self)
 {
 	AuthProviderObject *auth_provider = (AuthProviderObject *)self;
 	Py_XDECREF(auth_provider->callback);
+	auth_provider->callback = NULL;
 	apr_pool_destroy(auth_provider->pool);
 	PyObject_Del(self);
 }
@@ -2105,50 +2147,49 @@ static PyObject *auth_init(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 	char *kwnames[] = { "providers", NULL };
 	apr_array_header_t *c_providers;
 	svn_auth_provider_object_t **el;
-	PyObject *providers = Py_None;
+	PyObject *providers;
 	AuthObject *ret;
 	int i;
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|O", kwnames, &providers))
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O", kwnames, &providers))
 		return NULL;
 
 	ret = PyObject_New(AuthObject, &Auth_Type);
 	if (ret == NULL)
 		return NULL;
 
-	if (!PySequence_Check(providers)) {
-		PyErr_SetString(PyExc_TypeError, "Auth providers should be a sequence");
-		PyObject_Del(ret);
-		return NULL;
-	}
+	ret->providers = NULL;
 
 	ret->pool = Pool(NULL);
 	if (ret->pool == NULL) {
 		PyErr_NoMemory();
-		apr_pool_destroy(ret->pool);
-		PyObject_Del(ret);
+		Py_DECREF(ret);
 		return NULL;
 	}
 
-	ret->providers = providers;
-	Py_INCREF(providers);
+	if (!PySequence_Check(providers)) {
+		PyErr_SetString(PyExc_TypeError, "Auth providers should be a sequence");
+		Py_DECREF(ret);
+		return NULL;
+	}
 
-	c_providers = apr_array_make(ret->pool, PySequence_Size(providers), sizeof(svn_auth_provider_object_t *));
+	Py_INCREF(providers);
+	ret->providers = providers;
+
+	c_providers = apr_array_make(ret->pool, PySequence_Size(providers),
+								 sizeof(svn_auth_provider_object_t *));
 	if (c_providers == NULL) {
 		PyErr_NoMemory();
-		apr_pool_destroy(ret->pool);
-		PyObject_Del(ret);
+		Py_DECREF(ret);
 		return NULL;
 	}
 	for (i = 0; i < PySequence_Size(providers); i++) {
 		AuthProviderObject *provider;
 		el = (svn_auth_provider_object_t **)apr_array_push(c_providers);
-		/* FIXME: Check that provider is indeed a AuthProviderObject object */
 		provider = (AuthProviderObject *)PySequence_GetItem(providers, i);
 		if (!PyObject_TypeCheck(provider, &AuthProvider_Type)) {
 			PyErr_SetString(PyExc_TypeError, "Invalid auth provider");
-			apr_pool_destroy(ret->pool);
-			PyObject_Del(ret);
+			Py_DECREF(ret);
 			return NULL;
 		}
 		*el = provider->provider;
@@ -2163,7 +2204,7 @@ static PyObject *auth_set_parameter(PyObject *self, PyObject *args)
 	char *name;
 	PyObject *value;
 	void *vvalue;
-	if (!PyArg_ParseTuple(args, "sO", &name, &value))
+	if (!PyArg_ParseTuple(args, "sO:set_parameter", &name, &value))
 		return NULL;
 
 	if (!strcmp(name, SVN_AUTH_PARAM_SSL_SERVER_FAILURES)) {
@@ -2191,7 +2232,7 @@ static PyObject *auth_get_parameter(PyObject *self, PyObject *args)
 	const void *value;
 	AuthObject *auth = (AuthObject *)self;
 
-	if (!PyArg_ParseTuple(args, "s", &name))
+	if (!PyArg_ParseTuple(args, "s:get_parameter", &name))
 		return NULL;
 
 	value = svn_auth_get_parameter(auth->auth_baton, name);
@@ -2225,7 +2266,7 @@ static PyObject *auth_first_credentials(PyObject *self, PyObject *args)
 	CredentialsIterObject *ret;
 	svn_auth_iterstate_t *state;
 
-	if (!PyArg_ParseTuple(args, "ss", &cred_kind, &realmstring))
+	if (!PyArg_ParseTuple(args, "ss:credentials", &cred_kind, &realmstring))
 		return NULL;
 
 	pool = Pool(NULL);
@@ -2364,6 +2405,7 @@ static void auth_dealloc(PyObject *self)
 	AuthObject *auth = (AuthObject *)self;
 	apr_pool_destroy(auth->pool);
 	Py_XDECREF(auth->providers);
+	PyObject_Del(auth);
 }
 
 static PyTypeObject Auth_Type = {
@@ -2490,7 +2532,8 @@ static PyObject *get_username_prompt_provider(PyObject *self, PyObject *args)
 	AuthProviderObject *auth;
 	PyObject *prompt_func;
 	int retry_limit;
-	if (!PyArg_ParseTuple(args, "Oi", &prompt_func, &retry_limit))
+	if (!PyArg_ParseTuple(args, "Oi:get_username_prompt_provider",
+			  &prompt_func, &retry_limit))
 		return NULL;
 	auth = PyObject_New(AuthProviderObject, &AuthProvider_Type);
 	if (auth == NULL)
@@ -2500,7 +2543,8 @@ static PyObject *get_username_prompt_provider(PyObject *self, PyObject *args)
 		return NULL;
 	Py_INCREF(prompt_func);
 	auth->callback = prompt_func;
-	svn_auth_get_username_prompt_provider(&auth->provider, py_username_prompt, (void *)prompt_func, retry_limit, auth->pool);
+	svn_auth_get_username_prompt_provider(&auth->provider, py_username_prompt,
+		 (void *)prompt_func, retry_limit, auth->pool);
 	return (PyObject *)auth;
 }
 
@@ -2826,26 +2870,34 @@ static PyObject *get_simple_provider(PyObject *self, PyObject *args)
 {
 	AuthProviderObject *auth;
 	PyObject *callback = Py_None;
+	apr_pool_t *pool;
 
-	if (!PyArg_ParseTuple(args, "|O", &callback))
+	if (!PyArg_ParseTuple(args, "|O:get_simple_provider", &callback))
 		return NULL;
 
+	pool = Pool(NULL);
+	if (pool == NULL)
+		return NULL;
 	auth = PyObject_New(AuthProviderObject, &AuthProvider_Type);
-	if (auth == NULL)
-		return NULL;
-	auth->pool = Pool(NULL);
-	if (auth->pool == NULL)
+	if (auth == NULL) {
+		apr_pool_destroy(pool);
 		return NULL;
+	}
+	auth->pool = pool;
 #if ONLY_SINCE_SVN(1, 6)
 	Py_INCREF(callback);
 	auth->callback = callback;
-	svn_auth_get_simple_provider2(&auth->provider, py_cb_get_simple_provider_prompt, callback, auth->pool);
+	svn_auth_get_simple_provider2(&auth->provider,
+		  py_cb_get_simple_provider_prompt, auth->callback, auth->pool);
 #else
+	auth->callback = NULL;
+	auth->provider = NULL;
 	if (callback != Py_None) {
-		PyErr_SetString(PyExc_NotImplementedError, "callback not supported with svn < 1.6");
+		PyErr_SetString(PyExc_NotImplementedError,
+			"callback not supported with svn < 1.6");
+		Py_DECREF(auth);
 		return NULL;
 	}
-	auth->callback = NULL;
 	svn_auth_get_simple_provider(&auth->provider, auth->pool);
 #endif
 	return (PyObject *)auth;
@@ -3114,8 +3166,6 @@ void init_ra(void)
 	if (PyType_Ready(&LogIterator_Type) < 0)
 		return;
 
-	initeditor();
-
 	apr_initialize();
 	pool = Pool(NULL);
 	if (pool == NULL)
