@@ -32,9 +32,16 @@ typedef struct {
 	apr_pool_t *pool;
 	void (*done_cb) (void *baton);
 	void *done_baton;
+	bool done;
 	PyObject *commit_callback;
 } EditorObject;
 
+static PyObject *py_editor_ctx_enter(PyObject *self)
+{
+	Py_INCREF(self);
+	return self;
+}
+
 PyObject *new_editor_object(const svn_delta_editor_t *editor, void *baton,
 							apr_pool_t *pool, PyTypeObject *type,
 							void (*done_cb) (void *), void *done_baton, PyObject *commit_callback)
@@ -46,6 +53,7 @@ PyObject *new_editor_object(const svn_delta_editor_t *editor, void *baton,
 	obj->baton = baton;
 	obj->pool = pool;
 	obj->done_cb = done_cb;
+	obj->done = false;
 	obj->done_baton = done_baton;
 	obj->commit_callback = commit_callback;
 	return (PyObject *)obj;
@@ -55,8 +63,10 @@ static void py_editor_dealloc(PyObject *self)
 {
 	EditorObject *editor = (EditorObject *)self;
 	Py_XDECREF(editor->commit_callback);
-	apr_pool_destroy(editor->pool);
-	editor->pool = NULL;
+	if (editor->pool != NULL) {
+		apr_pool_destroy(editor->pool);
+		editor->pool = NULL;
+	}
 	PyObject_Del(self);
 }
 
@@ -127,7 +137,9 @@ static PyObject *txdelta_call(PyObject *self, PyObject *args, PyObject *kwargs)
 	Py_BEGIN_ALLOW_THREADS
 	error = obj->txdelta_handler(&window, obj->txdelta_baton);
 	Py_END_ALLOW_THREADS
-	if (!check_error(error)) {
+	if (error != NULL) {
+		handle_svn_error(error);
+		svn_error_clear(error);
 		free(ops);
 		return NULL;
 	}
@@ -137,6 +149,11 @@ static PyObject *txdelta_call(PyObject *self, PyObject *args, PyObject *kwargs)
 	Py_RETURN_NONE;
 }
 
+static void py_txdelta_window_handler_dealloc(PyObject *self)
+{
+	PyObject_Del(self);
+}
+
 PyTypeObject TxDeltaWindowHandler_Type = {
 	PyObject_HEAD_INIT(NULL) 0,
 	"_ra.TxDeltaWindowHandler", /*	const char *tp_name;  For printing, in format "<module>.<name>" */
@@ -145,7 +162,7 @@ PyTypeObject TxDeltaWindowHandler_Type = {
 	
 	/* Methods to implement standard operations */
 	
-	NULL, /* destructor tp_dealloc;	(done in initeditor()) */
+	py_txdelta_window_handler_dealloc, /* destructor tp_dealloc; */
 	NULL, /*	printfunc tp_print;	*/
 	NULL, /*	getattrfunc tp_getattr;	*/
 	NULL, /*	setattrfunc tp_setattr;	*/
@@ -173,13 +190,14 @@ static PyObject *py_file_editor_apply_textdelta(PyObject *self, PyObject *args)
 	void *txdelta_baton;
 	TxDeltaWindowHandlerObject *py_txdelta;
 
-	if (!FileEditor_Check(self)) {
-		PyErr_BadArgument();
+	if (!PyArg_ParseTuple(args, "|z", &c_base_checksum))
+		return NULL;
+
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "file editor already closed");
 		return NULL;
 	}
 
-	if (!PyArg_ParseTuple(args, "|z", &c_base_checksum))
-		return NULL;
 	RUN_SVN(editor->editor->apply_textdelta(editor->baton,
 				c_base_checksum, editor->pool, 
 				&txdelta_handler, &txdelta_baton));
@@ -193,21 +211,22 @@ static PyObject *py_file_editor_change_prop(PyObject *self, PyObject *args)
 {
 	EditorObject *editor = (EditorObject *)self;
 	char *name;
-   	svn_string_t c_value;
+	svn_string_t c_value;
 	int vallen;
 
-	if (!FileEditor_Check(self)) {
-		PyErr_BadArgument();
-		return NULL;
-	}
-
 	if (!PyArg_ParseTuple(args, "sz#", &name, &c_value.data, &vallen))
 		return NULL;
 
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "file editor already closed");
+		return NULL;
+	}
+
 	c_value.len = vallen;
 
 	RUN_SVN(editor->editor->change_file_prop(editor->baton, name, 
 				(c_value.data == NULL)?NULL:&c_value, editor->pool));
+
 	Py_RETURN_NONE;
 }
 
@@ -216,22 +235,54 @@ static PyObject *py_file_editor_close(PyObject *self, PyObject *args)
 	EditorObject *editor = (EditorObject *)self;
 	char *c_checksum = NULL;
 
-	if (!FileEditor_Check(self)) {
-		PyErr_BadArgument();
+	if (!PyArg_ParseTuple(args, "|z", &c_checksum))
+		return NULL;
+
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "file editor was already closed");
 		return NULL;
 	}
 
-	if (!PyArg_ParseTuple(args, "|z", &c_checksum))
-		return NULL;
 	RUN_SVN(editor->editor->close_file(editor->baton, c_checksum, 
 					editor->pool));
+
+	editor->done = true;
+	apr_pool_destroy(editor->pool);
+	editor->pool = NULL;
+
 	Py_RETURN_NONE;
 }
 
+static PyObject *py_file_editor_ctx_enter(PyObject *self)
+{
+	Py_INCREF(self);
+	return self;
+}
+
+static PyObject *py_file_editor_ctx_exit(PyObject *self, PyObject *args)
+{
+	EditorObject *editor = (EditorObject *)self;
+
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "file editor already closed");
+		return NULL;
+	}
+
+	RUN_SVN(editor->editor->close_file(editor->baton, NULL, editor->pool));
+
+	editor->done = true;
+	apr_pool_destroy(editor->pool);
+	editor->pool = NULL;
+
+	Py_RETURN_FALSE;
+}
+
 static PyMethodDef py_file_editor_methods[] = {
 	{ "change_prop", py_file_editor_change_prop, METH_VARARGS, NULL },
 	{ "close", py_file_editor_close, METH_VARARGS, NULL },
 	{ "apply_textdelta", py_file_editor_apply_textdelta, METH_VARARGS, NULL },
+	{ "__enter__", (PyCFunction)py_file_editor_ctx_enter, METH_NOARGS, NULL },
+	{ "__exit__", py_file_editor_ctx_exit, METH_VARARGS, NULL },
 	{ NULL }
 };
 
@@ -243,7 +294,7 @@ PyTypeObject FileEditor_Type = {
 	
 	/* Methods to implement standard operations */
 	
-	NULL, /*	destructor tp_dealloc; (Done in initeditor()) 	*/
+	py_editor_dealloc, /*	destructor tp_dealloc; 	*/
 	NULL, /*	printfunc tp_print;	*/
 	NULL, /*	getattrfunc tp_getattr;	*/
 	NULL, /*	setattrfunc tp_setattr;	*/
@@ -301,14 +352,14 @@ static PyObject *py_dir_editor_delete_entry(PyObject *self, PyObject *args)
 	char *path; 
 	svn_revnum_t revision = -1;
 
-	if (!DirectoryEditor_Check(self)) {
-		PyErr_BadArgument();
-		return NULL;
-	}
-
 	if (!PyArg_ParseTuple(args, "s|l", &path, &revision))
 		return NULL;
 
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "directory editor already closed");
+		return NULL;
+	}
+
 	RUN_SVN(editor->editor->delete_entry(svn_path_canonicalize(path, editor->pool),
 										 revision, editor->baton, editor->pool));
 
@@ -324,20 +375,20 @@ static PyObject *py_dir_editor_add_directory(PyObject *self, PyObject *args)
 	EditorObject *editor = (EditorObject *)self;
 	apr_pool_t *subpool;
 
-	if (!DirectoryEditor_Check(self)) {
-		PyErr_BadArgument();
-		return NULL;
-	}
-
 	if (!PyArg_ParseTuple(args, "s|zl", &path, &copyfrom_path, &copyfrom_rev))
 		return NULL;
 
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "directory editor already closed");
+		return NULL;
+	}
+
 	RUN_SVN(editor->editor->add_directory(
 		svn_path_canonicalize(path, editor->pool), editor->baton,
 		copyfrom_path == NULL?NULL:svn_path_canonicalize(copyfrom_path, editor->pool), 
 		copyfrom_rev, editor->pool, &child_baton));
 
-	subpool = Pool(NULL);
+	subpool = Pool(editor->pool);
 	if (subpool == NULL)
 		return NULL;
 
@@ -353,14 +404,14 @@ static PyObject *py_dir_editor_open_directory(PyObject *self, PyObject *args)
 	void *child_baton;
 	apr_pool_t *subpool;
 
-	if (!DirectoryEditor_Check(self)) {
-		PyErr_BadArgument();
-		return NULL;
-	}
-
 	if (!PyArg_ParseTuple(args, "s|l", &path, &base_revision))
 		return NULL;
 
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "directory editor already closed");
+		return NULL;
+	}
+
 	RUN_SVN(editor->editor->open_directory(
 		svn_path_canonicalize(path, editor->pool), editor->baton,
 					base_revision, editor->pool, &child_baton));
@@ -380,14 +431,14 @@ static PyObject *py_dir_editor_change_prop(PyObject *self, PyObject *args)
 	EditorObject *editor = (EditorObject *)self;
 	int vallen;
 
-	if (!DirectoryEditor_Check(self)) {
-		PyErr_BadArgument();
-		return NULL;
-	}
-
 	if (!PyArg_ParseTuple(args, "sz#", &name, &c_value.data, &vallen))
 		return NULL;
 
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "directory editor already closed");
+		return NULL;
+	}
+
 	c_value.len = vallen;
 
 	RUN_SVN(editor->editor->change_dir_prop(editor->baton, name, 
@@ -400,13 +451,17 @@ static PyObject *py_dir_editor_close(PyObject *self)
 {
 	EditorObject *editor = (EditorObject *)self;
 
-	if (!DirectoryEditor_Check(self)) {
-		PyErr_BadArgument();
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "directory editor was already closed");
 		return NULL;
 	}
 
 	RUN_SVN(editor->editor->close_directory(editor->baton, editor->pool));
 
+	editor->done = true;
+	apr_pool_destroy(editor->pool);
+	editor->pool = NULL;
+
 	Py_RETURN_NONE;
 }
 
@@ -415,15 +470,14 @@ static PyObject *py_dir_editor_absent_directory(PyObject *self, PyObject *args)
 	char *path;
 	EditorObject *editor = (EditorObject *)self;
 
-	if (!Editor_Check(self)) {
-		PyErr_BadArgument();
+	if (!PyArg_ParseTuple(args, "s", &path))
+		return NULL;
+
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "directory editor already closed");
 		return NULL;
 	}
 
-
-	if (!PyArg_ParseTuple(args, "s", &path))
-		return NULL;
-	
 	RUN_SVN(editor->editor->absent_directory(
 		svn_path_canonicalize(path, editor->pool), editor->baton, editor->pool));
 
@@ -438,14 +492,14 @@ static PyObject *py_dir_editor_add_file(PyObject *self, PyObject *args)
 	EditorObject *editor = (EditorObject *)self;
 	apr_pool_t *subpool;
 
-	if (!DirectoryEditor_Check(self)) {
-		PyErr_BadArgument();
-		return NULL;
-	}
-
 	if (!PyArg_ParseTuple(args, "s|zl", &path, &copy_path, &copy_rev))
 		return NULL;
 
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "directory editor already closed");
+		return NULL;
+	}
+
 	RUN_SVN(editor->editor->add_file(svn_path_canonicalize(path, editor->pool),
 		editor->baton, 
 		copy_path == NULL?NULL:svn_path_canonicalize(copy_path, editor->pool),
@@ -467,14 +521,14 @@ static PyObject *py_dir_editor_open_file(PyObject *self, PyObject *args)
 	EditorObject *editor = (EditorObject *)self;
 	apr_pool_t *subpool;
 
-	if (!DirectoryEditor_Check(self)) {
-		PyErr_BadArgument();
-		return NULL;
-	}
-
 	if (!PyArg_ParseTuple(args, "s|l", &path, &base_revision))
 		return NULL;
 
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "directory editor already closed");
+		return NULL;
+	}
+
 	RUN_SVN(editor->editor->open_file(svn_path_canonicalize(path, editor->pool),
 									  editor->baton, base_revision, 
 									  editor->pool, &file_baton));
@@ -492,13 +546,14 @@ static PyObject *py_dir_editor_absent_file(PyObject *self, PyObject *args)
 	char *path;
 	EditorObject *editor = (EditorObject *)self;
 
-	if (!DirectoryEditor_Check(self)) {
-		PyErr_BadArgument();
+	if (!PyArg_ParseTuple(args, "s", &path))
+		return NULL;
+
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "directory editor already closed");
 		return NULL;
 	}
 
-	if (!PyArg_ParseTuple(args, "s", &path))
-		return NULL;
 
 	RUN_SVN(editor->editor->absent_file(
 		svn_path_canonicalize(path, editor->pool), editor->baton, editor->pool));
@@ -506,8 +561,34 @@ static PyObject *py_dir_editor_absent_file(PyObject *self, PyObject *args)
 	Py_RETURN_NONE;
 }
 
+static PyObject *py_dir_editor_ctx_enter(PyObject *self)
+{
+	Py_INCREF(self);
+	return self;
+}
+
+static PyObject *py_dir_editor_ctx_exit(PyObject *self, PyObject *args)
+{
+	EditorObject *editor = (EditorObject *)self;
+
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "directory editor already closed");
+		return NULL;
+	}
+
+	RUN_SVN(editor->editor->close_directory(editor->baton, editor->pool));
+
+	editor->done = true;
+	apr_pool_destroy(editor->pool);
+	editor->pool = NULL;
+
+	Py_RETURN_FALSE;
+}
+
 static PyMethodDef py_dir_editor_methods[] = {
-	{ "absent_file", py_dir_editor_absent_file, METH_VARARGS, NULL },
+	{ "absent_file", py_dir_editor_absent_file, METH_VARARGS,
+		"S.absent_file(path)\n\n"
+		"Indicate a file is not present." },
 	{ "absent_directory", py_dir_editor_absent_directory, METH_VARARGS, NULL },
 	{ "delete_entry", py_dir_editor_delete_entry, METH_VARARGS, NULL },
 	{ "add_file", py_dir_editor_add_file, METH_VARARGS, NULL },
@@ -516,7 +597,8 @@ static PyMethodDef py_dir_editor_methods[] = {
 	{ "open_directory", py_dir_editor_open_directory, METH_VARARGS, NULL },
 	{ "close", (PyCFunction)py_dir_editor_close, METH_NOARGS, NULL },
 	{ "change_prop", py_dir_editor_change_prop, METH_VARARGS, NULL },
-
+	{ "__enter__", (PyCFunction)py_dir_editor_ctx_enter, METH_NOARGS, NULL },
+	{ "__exit__", py_dir_editor_ctx_exit, METH_VARARGS, NULL },
 	{ NULL, }
 };
 
@@ -528,7 +610,7 @@ PyTypeObject DirectoryEditor_Type = {
 	
 	/* Methods to implement standard operations */
 	
-	NULL, /* destructor tp_dealloc;	(Done in initeditor()) */
+	py_editor_dealloc, /* destructor tp_dealloc;  */
 	NULL, /*	printfunc tp_print;	*/
 	NULL, /*	getattrfunc tp_getattr;	*/
 	NULL, /*	setattrfunc tp_setattr;	*/
@@ -586,20 +668,20 @@ static PyObject *py_editor_set_target_revision(PyObject *self, PyObject *args)
 	svn_revnum_t target_revision;
 	EditorObject *editor = (EditorObject *)self;
 
-	if (!Editor_Check(self)) {
-		PyErr_BadArgument();
-		return NULL;
-	}
-
 	if (!PyArg_ParseTuple(args, "l", &target_revision))
 		return NULL;
 
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "Editor already closed/aborted");
+		return NULL;
+	}
+
 	RUN_SVN(editor->editor->set_target_revision(editor->baton,
 					target_revision, editor->pool));
 
 	Py_RETURN_NONE;
 }
-	
+
 static PyObject *py_editor_open_root(PyObject *self, PyObject *args)
 {
 	svn_revnum_t base_revision=-1;
@@ -607,14 +689,14 @@ static PyObject *py_editor_open_root(PyObject *self, PyObject *args)
 	EditorObject *editor = (EditorObject *)self;
 	apr_pool_t *subpool;
 
-	if (!Editor_Check(self)) {
-		PyErr_BadArgument();
-		return NULL;
-	}
-
 	if (!PyArg_ParseTuple(args, "|l:open_root", &base_revision))
 		return NULL;
 
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "Editor already closed/aborted");
+		return NULL;
+	}
+
 	RUN_SVN(editor->editor->open_root(editor->baton, base_revision,
 					editor->pool, &root_baton));
 
@@ -630,13 +712,17 @@ static PyObject *py_editor_close(PyObject *self)
 {
 	EditorObject *editor = (EditorObject *)self;
 
-	if (!Editor_Check(self)) {
-		PyErr_BadArgument();
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "Editor already closed/aborted");
 		return NULL;
 	}
 
 	RUN_SVN(editor->editor->close_edit(editor->baton, editor->pool));
 
+	editor->done = true;
+	apr_pool_destroy(editor->pool);
+	editor->pool = NULL;
+
 	if (editor->done_cb != NULL)
 		editor->done_cb(editor->done_baton);
 
@@ -647,19 +733,48 @@ static PyObject *py_editor_abort(PyObject *self)
 {
 	EditorObject *editor = (EditorObject *)self;
 
-	if (!Editor_Check(self)) {
-		PyErr_BadArgument();
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "Editor already closed/aborted");
 		return NULL;
 	}
 
 	RUN_SVN(editor->editor->abort_edit(editor->baton, editor->pool));
 
+	editor->done = true;
+	apr_pool_destroy(editor->pool);
+	editor->pool = NULL;
+
 	if (editor->done_cb != NULL)
 		editor->done_cb(editor->done_baton);
-	
+
 	Py_RETURN_NONE;
 }
 
+static PyObject *py_editor_ctx_exit(PyObject *self, PyObject *args)
+{
+	EditorObject *editor = (EditorObject *)self;
+	PyObject *exc_type, *exc_val, *exc_tb;
+
+	if (!PyArg_ParseTuple(args, "OOO", &exc_type, &exc_val, &exc_tb))
+		return NULL;
+
+	if (editor->done) {
+		PyErr_SetString(PyExc_RuntimeError, "Editor already closed/aborted");
+		return NULL;
+	}
+
+	if (exc_type != Py_None) {
+		RUN_SVN(editor->editor->abort_edit(editor->baton, editor->pool));
+	} else {
+		RUN_SVN(editor->editor->close_edit(editor->baton, editor->pool));
+	}
+
+	if (editor->done_cb != NULL)
+		editor->done_cb(editor->done_baton);
+
+	Py_RETURN_FALSE;
+}
+
 static PyMethodDef py_editor_methods[] = { 
 	{ "abort", (PyCFunction)py_editor_abort, METH_NOARGS, 
 		"S.abort()\n"
@@ -673,7 +788,9 @@ static PyMethodDef py_editor_methods[] = {
 	{ "set_target_revision", py_editor_set_target_revision, METH_VARARGS,
 		"S.set_target_revision(target_revision)\n"
 		"Set the target revision created by the reported revision."},
-	{ NULL, }
+	{ "__enter__", (PyCFunction)py_editor_ctx_enter, METH_NOARGS, NULL },
+	{ "__exit__", py_editor_ctx_exit, METH_VARARGS, NULL },
+	{ NULL }
 };
 
 PyTypeObject Editor_Type = { 
@@ -737,13 +854,6 @@ PyTypeObject Editor_Type = {
 };
 
 
-void initeditor(void)
-{
-    TxDeltaWindowHandler_Type.tp_dealloc = (destructor)PyObject_Del;
-    FileEditor_Type.tp_dealloc = (destructor)PyObject_Del;
-    DirectoryEditor_Type.tp_dealloc = (destructor)PyObject_Del;
-}
-
 static svn_error_t *py_cb_editor_set_target_revision(void *edit_baton, svn_revnum_t target_revision, apr_pool_t *pool)
 {
 	PyObject *self = (PyObject *)edit_baton, *ret;
