@@ -16,20 +16,28 @@
 """Subversion ra library tests."""
 
 from cStringIO import StringIO
-from unittest import TestCase
 
 from subvertpy import (
-    NODE_DIR, NODE_NONE,
+    NODE_DIR, NODE_NONE, NODE_UNKNOWN,
     SubversionException,
     ra,
     )
-from subvertpy.tests import SubversionTestCase
+from subvertpy.tests import (
+    SubversionTestCase,
+    TestCase,
+    )
 
 class VersionTest(TestCase):
 
     def test_version_length(self):
         self.assertEquals(4, len(ra.version()))
 
+    def test_api_version_length(self):
+        self.assertEquals(4, len(ra.api_version()))
+
+    def test_api_version_later_same(self):
+        self.assertTrue(ra.api_version() <= ra.version())
+
 
 class TestRemoteAccessUnknown(TestCase):
 
@@ -70,23 +78,36 @@ class TestRemoteAccess(SubversionTestCase):
     def test_get_repos_root(self):
         self.assertEqual(self.repos_url, self.ra.get_repos_root())
 
+    def test_get_url(self):
+        if ra.api_version() < (1, 5):
+            self.assertRaises(NotImplementedError, self.ra.get_url)
+        else:
+            self.assertEqual(self.repos_url, self.ra.get_url())
+
     def test_reparent(self):
         self.ra.reparent(self.repos_url)
 
     def test_has_capability(self):
-        try:
+        if ra.api_version() < (1, 5):
+            self.assertRaises(NotImplementedError, self.ra.has_capability, "FOO")
+        else:
             self.assertRaises(SubversionException, self.ra.has_capability, "FOO")
-        except NotImplementedError:
-            # svn < 1.5
-            return
 
     def test_get_dir(self):
         ret = self.ra.get_dir("", 0)
         self.assertIsInstance(ret, tuple)
 
+    def test_get_dir_leading_slash(self):
         ret = self.ra.get_dir("/", 0)
         self.assertIsInstance(ret, tuple)
 
+    def test_get_dir_kind(self):
+        self.do_commit()
+        (dirents, fetch_rev, props) = self.ra.get_dir("/", 1, fields=ra.DIRENT_KIND)
+        self.assertIsInstance(props, dict)
+        self.assertEquals(1, fetch_rev)
+        self.assertEquals(NODE_DIR, dirents["foo"]["kind"])
+
     def test_change_rev_prop(self):
         self.do_commit()
         self.ra.change_rev_prop(1, "foo", "bar")
@@ -115,6 +136,44 @@ class TestRemoteAccess(SubversionTestCase):
         reporter = self.ra.do_diff(1, "", self.ra.get_repos_root(), MyEditor())
         reporter.set_path("", 0, True)
         reporter.finish()
+        self.assertRaises(RuntimeError, reporter.finish)
+        self.assertRaises(RuntimeError, reporter.set_path, "", 0, True)
+
+    def test_iter_log_invalid(self):
+        self.assertRaises(SubversionException, list, self.ra.iter_log(
+                ["idontexist"], 0, 0, revprops=["svn:date", "svn:author", "svn:log"]))
+        self.assertRaises(SubversionException, list, self.ra.iter_log(
+                [""], 0, 1000, revprops=["svn:date", "svn:author", "svn:log"]))
+
+    def test_iter_log(self):
+        def check_results(returned):
+            self.assertEquals(2, len(returned))
+            self.assert_(len(returned[0]) in (3,4))
+            if len(returned[0]) == 3:
+                (paths, revnum, props) = returned[0]
+            else:
+                (paths, revnum, props, has_children) = returned[0]
+            self.assertEquals(None, paths)
+            self.assertEquals(revnum, 0)
+            self.assertEquals(["svn:date"], props.keys())
+            if len(returned[1]) == 3:
+                (paths, revnum, props) = returned[1]
+            else:
+                (paths, revnum, props, has_children) = returned[1]
+            if ra.api_version() < (1, 6):
+                self.assertEquals({'/foo': ('A', None, -1, NODE_UNKNOWN)}, paths)
+            else:
+                self.assertEquals({'/foo': ('A', None, -1, NODE_DIR)}, paths)
+            self.assertEquals(revnum, 1)
+            self.assertEquals(set(["svn:date", "svn:author", "svn:log"]), 
+                              set(props.keys()))
+        returned = list(self.ra.iter_log([""], 0, 0,
+            revprops=["svn:date", "svn:author", "svn:log"]))
+        self.assertEquals(1, len(returned))
+        self.do_commit()
+        returned = list(self.ra.iter_log(None, 0, 1, discover_changed_paths=True, 
+            strict_node_history=False, revprops=["svn:date", "svn:author", "svn:log"]))
+        check_results(returned)
 
     def test_get_log(self):
         returned = []
@@ -146,11 +205,30 @@ class TestRemoteAccess(SubversionTestCase):
                         strict_node_history=False, revprops=["svn:date", "svn:author", "svn:log"])
         check_results(returned)
 
+    def test_get_log_cancel(self):
+        def cb(*args):
+            raise KeyError
+        self.do_commit()
+        self.assertRaises(KeyError,
+            self.ra.get_log, cb, [""], 0, 0, revprops=["svn:date", "svn:author", "svn:log"])
+
+    def test_get_commit_editor_double_close(self):
+        def mycb(*args):
+            pass
+        editor = self.ra.get_commit_editor({"svn:log": "foo"}, mycb)
+        dir = editor.open_root()
+        dir.close()
+        self.assertRaises(RuntimeError, dir.close)
+        editor.close()
+        self.assertRaises(RuntimeError, editor.close)
+        self.assertRaises(RuntimeError, editor.abort)
+
     def test_get_commit_editor_busy(self):
         def mycb(rev):
             pass
         editor = self.ra.get_commit_editor({"svn:log": "foo"}, mycb)
-        self.assertRaises(ra.BusyException, self.ra.get_commit_editor, {"svn:log": "foo"}, mycb)
+        self.assertRaises(ra.BusyException, self.ra.get_commit_editor,
+            {"svn:log": "foo"}, mycb)
         editor.abort()
 
     def test_get_commit_editor_custom_revprops(self):
@@ -166,6 +244,18 @@ class TestRemoteAccess(SubversionTestCase):
 
         self.assertEquals(set(['bar:foo', 'svn:author', 'svn:custom:blie', 'svn:date', 'svn:log']), set(self.ra.rev_proplist(1).keys()))
 
+    def test_get_commit_editor_context_manager(self):
+        def mycb(paths, rev, revprops):
+            pass
+        editor = self.ra.get_commit_editor({"svn:log": "foo"}, mycb)
+        self.assertIs(editor, editor.__enter__())
+        dir = editor.open_root(0)
+        subdir = dir.add_directory("foo")
+        self.assertIs(subdir, subdir.__enter__())
+        subdir.__exit__(None, None, None)
+        dir.__exit__(None, None, None)
+        editor.__exit__(None, None, None)
+
     def test_get_commit_editor(self):
         def mycb(paths, rev, revprops):
             pass
@@ -187,14 +277,14 @@ class TestRemoteAccess(SubversionTestCase):
         f = cb.open_file("bar")
         f.change_prop("bla:bar", None)
         cb.close()
-        
+
         stream = StringIO()
         props = self.ra.get_file("bar", stream, 1)[1]
         self.assertEquals("blie", props.get("bla:bar"))
         stream = StringIO()
         props = self.ra.get_file("bar", stream, 2)[1]
         self.assertIs(None, props.get("bla:bar"))
-    
+
     def test_get_file_revs(self):
         cb = self.commit_editor()
         cb.add_file("bar").modify("a")
@@ -205,10 +295,10 @@ class TestRemoteAccess(SubversionTestCase):
         f.modify("b")
         f.change_prop("bla", "bloe")
         cb.close()
-        
+
         rets = []
 
-        def handle(path, rev, props):
+        def handle(path, rev, props, from_merge=None):
             rets.append((path, rev, props))
 
         self.ra.get_file_revs("bar", 1, 2, handle)
